@@ -73,12 +73,10 @@ SOFTWARE.
 int
 SProcXGetSelectedExtensionEvents(ClientPtr client)
 {
-    char n;
-
     REQUEST(xGetSelectedExtensionEventsReq);
-    swaps(&stuff->length, n);
+    swaps(&stuff->length);
     REQUEST_SIZE_MATCH(xGetSelectedExtensionEventsReq);
-    swapl(&stuff->window, n);
+    swapl(&stuff->window);
     return (ProcXGetSelectedExtensionEvents(client));
 }
 
@@ -118,13 +116,13 @@ ProcXGetSelectedExtensionEvents(ClientPtr client)
     if ((pOthers = wOtherInputMasks(pWin)) != 0) {
 	for (others = pOthers->inputClients; others; others = others->next)
 	    for (i = 0; i < EMASKSIZE; i++)
-		tclient = ClassFromMask(NULL, others->mask[i], i,
+		ClassFromMask(NULL, others->mask[i], i,
 					&rep.all_clients_count, COUNT);
 
 	for (others = pOthers->inputClients; others; others = others->next)
 	    if (SameClient(others, client)) {
 		for (i = 0; i < EMASKSIZE; i++)
-		    tclient = ClassFromMask(NULL, others->mask[i], i,
+		    ClassFromMask(NULL, others->mask[i], i,
 					    &rep.this_client_count, COUNT);
 		break;
 	    }
@@ -132,7 +130,7 @@ ProcXGetSelectedExtensionEvents(ClientPtr client)
 	total_length = (rep.all_clients_count + rep.this_client_count) *
 	    sizeof(XEventClass);
 	rep.length = bytes_to_int32(total_length);
-	buf = (XEventClass *) xalloc(total_length);
+	buf = (XEventClass *) malloc(total_length);
 
 	tclient = buf;
 	aclient = buf + rep.this_client_count;
@@ -152,8 +150,8 @@ ProcXGetSelectedExtensionEvents(ClientPtr client)
     if (total_length) {
 	client->pSwapReplyFunc = (ReplySwapPtr) Swap32Write;
 	WriteSwappedDataToClient(client, total_length, buf);
-	xfree(buf);
     }
+    free(buf);
     return Success;
 }
 
@@ -168,11 +166,9 @@ void
 SRepXGetSelectedExtensionEvents(ClientPtr client, int size,
 				xGetSelectedExtensionEventsReply * rep)
 {
-    char n;
-
-    swaps(&rep->sequenceNumber, n);
-    swapl(&rep->length, n);
-    swaps(&rep->this_client_count, n);
-    swaps(&rep->all_clients_count, n);
+    swaps(&rep->sequenceNumber);
+    swapl(&rep->length);
+    swaps(&rep->this_client_count);
+    swaps(&rep->all_clients_count);
     WriteToClient(client, size, (char *)rep);
 }
