@@ -9,7 +9,6 @@
 #include "xf86_OSproc.h"
 
 /* pci stuff */
-#include "xf86PciInfo.h"
 #include "xf86Pci.h"
 
 #include "xf86cmap.h"
@@ -48,7 +47,6 @@ _X_EXPORT XF86ModuleData fbdevhwModuleData = {
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <string.h>
 
 /* -------------------------------------------------------------------- */
 /* our private data, and two functions to allocate/free this            */
@@ -100,8 +98,6 @@ fbdevHWFreeRec(ScrnInfoPtr pScrn)
 {
 	if (fbdevHWPrivateIndex < 0)
 		return;
-	if (FBDEVHWPTR(pScrn) == NULL)
-		return;
 	free(FBDEVHWPTR(pScrn));
 	FBDEVHWPTRLVAL(pScrn) = NULL;
 }
@@ -267,20 +263,20 @@ fbdev_open_pci(struct pci_device * pPci, char **namep)
     int	fd, i;
 
     for (i = 0; i < 8; i++) {
-	sprintf(filename, 
+	snprintf(filename, sizeof(filename),
 		"/sys/bus/pci/devices/%04x:%02x:%02x.%d/graphics/fb%d",
 		pPci->domain, pPci->bus, pPci->dev, pPci->func, i);
 
 	fd = open(filename, O_RDONLY, 0);
         if (fd < 0) {
-            sprintf(filename,
+            snprintf(filename, sizeof(filename),
                     "/sys/bus/pci/devices/%04x:%02x:%02x.%d/graphics:fb%d",
                     pPci->domain, pPci->bus, pPci->dev, pPci->func, i);
             fd = open(filename, O_RDONLY, 0);
         }
 	if (fd >= 0) {
 	    close(fd);
-	    sprintf(filename, "/dev/fb%d", i);
+	    snprintf(filename, sizeof(filename), "/dev/fb%d", i);
 
 	    fd = open(filename, O_RDWR, 0);
 	    if (fd != -1) {
