@@ -35,13 +35,7 @@
  */
 #include "winkeynames.h"
 
-
-/*
- * Include the standard ASCII keymap.
- *
- * This header declares a static KeySym array called 'map'.
- */
-#include "winkeymap.h"
+#define VK_FN 0xFF
 
 #define		WIN_KEYMAP_COLS		3
 
@@ -224,13 +218,13 @@ g_iKeyMap [] = {
   /* 170 */	0,		0,		0,
   /* 171 */	0,		0,		0,
   /* 172 */	0,		0,		0,
-  /* 173 */	0,		0,		0,
-  /* 174 */	0,		0,		0,
-  /* 175 */	0,		0,		0,
-  /* 176 */	0,		0,		0,
-  /* 177 */	0,		0,		0,
-  /* 178 */	0,		0,		0,
-  /* 179 */	0,		0,		0,
+  /* 173 */	VK_VOLUME_MUTE,	0,		KEY_Mute,
+  /* 174 */	VK_VOLUME_DOWN,	0,		KEY_AudioLower,
+  /* 175 */	VK_VOLUME_UP,	0,		KEY_AudioRaise,
+  /* 176 */	VK_MEDIA_NEXT_TRACK,	0,	KEY_NEXTSONG,
+  /* 177 */	VK_MEDIA_PREV_TRACK,	0,	KEY_PREVIOUSSONG,
+  /* 178 */	VK_MEDIA_STOP,	0,		KEY_STOPCD,
+  /* 179 */	VK_MEDIA_PLAY_PAUSE,	0,	KEY_PLAYPAUSE,
   /* 180 */	0,		0,		0,
   /* 181 */	0,		0,		0,
   /* 182 */	0,		0,		0,
@@ -274,7 +268,7 @@ g_iKeyMap [] = {
   /* 220 */	0,		0,		0,
   /* 221 */	0,		0,		0,
   /* 222 */	0,		0,		0,
-  /* 223 */	0,		0,		0,
+  /* 223 */	VK_OEM_8,	0,		KEY_RCtrl,  /* at least on Canadian Multilingual Standard layout */
   /* 224 */	0,		0,		0,
   /* 225 */	0,		0,		0,
   /* 226 */	0,		0,		0,
@@ -306,7 +300,7 @@ g_iKeyMap [] = {
   /* 252 */	0,		0,		0,
   /* 253 */	0,		0,		0,
   /* 254 */	0,		0,		0,
-  /* 255 */	0,		0,		0
+  /* 255 */	VK_FN,		0,		KEY_Fn  /* Most keyboards don't generate a scancode for Fn, but a few do... */
 };
 
 #endif /* WINKEYBD_H */
